# title: "EF and NPS Measurement Invariance and Growth Curve Modeling"
# authors: "Grace Goodwin, MS; & D. A. Briley, PhD"
# date: "2024-11-14"

## Load Packages
if (!require("tidyverse")) {install.packages("tidyverse"); require("tidyverse")}
if (!require("dplyr")) {install.packages("dplyr"); require("dplyr")}
if (!require("lavaan")) {install.packages("lavaan"); require("lavaan")}
if (!require("psych")) {install.packages("psych"); require("psych")}
if (!require("semPower")) {install.packages("semPower"); require("semPower")}
if (!require("DescTools")) {install.packages("DescTools"); require("DescTools")}
if (!require("moments")) {install.packages("moments"); require("moments")}
if (!require("gtools")) {install.packages("gtools"); require("gtools")}
if (!require("ggplot")) {install.packages("ggplot"); require("ggplot")}
if (!require("utils")) {install.packages("utils"); require("utils")}
if (!require("patchwork")) {install.packages("patchwork"); require("patchwork")}
if (!require("epiDisplay")) {install.packages("epiDisplay"); require("epiDisplay")}

## Read Data
df = readRDS("~/R Projects/CANE Lab Research/df.RDS")

## Measurement Invariance
################ Configural ################
#do trails, number sequencing, and letter fluency tests measure the same latent EF construct across time?
#we will be very explicit with our modeling here and use no defaults with the lavaan() function
#first loading NA across time (ldigbacc), freely estimating loadings for nums and letters
mi.configural<-'
#define latent variables, all loadings estimated
EF_0 =~ NA*DIGBACCT_W0 + UDSVERTN_W0S + TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + UDSVERTN_W1S + TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + UDSVERTN_W2S + TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + UDSVERTN_W3S + TRAILB_W3rS


#define indicator intercepts

DIGBACCT_W0 ~ 1
DIGBACCT_W1 ~ 1
DIGBACCT_W2 ~ 1
DIGBACCT_W3 ~ 1

UDSVERTN_W0S ~ 1
UDSVERTN_W1S ~ 1
UDSVERTN_W2S ~ 1
UDSVERTN_W3S ~ 1

TRAILB_W0rS ~ 1
TRAILB_W1rS ~ 1
TRAILB_W2rS ~ 1
TRAILB_W3rS ~ 1

#define indicator residual variance

DIGBACCT_W0 ~~ DIGBACCT_W0
DIGBACCT_W1 ~~ DIGBACCT_W1
DIGBACCT_W2 ~~ DIGBACCT_W2
DIGBACCT_W3 ~~ DIGBACCT_W3

UDSVERTN_W0S ~~ UDSVERTN_W0S
UDSVERTN_W1S ~~ UDSVERTN_W1S
UDSVERTN_W2S ~~ UDSVERTN_W2S
UDSVERTN_W3S ~~ UDSVERTN_W3S

TRAILB_W0rS ~~ TRAILB_W0rS
TRAILB_W1rS ~~ TRAILB_W1rS
TRAILB_W2rS ~~ TRAILB_W2rS
TRAILB_W3rS ~~ TRAILB_W3rS

#specify latent means and variance
#fix mean of latent factors to 0 (creates standardized latent factor)

EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1


#fix variance of latent factors to 1 (creates standardized latent factor)

EF_0 ~~ (1)*EF_0
EF_1 ~~ (1)*EF_1
EF_2 ~~ (1)*EF_2
EF_3 ~~ (1)*EF_3


#specify latent correlations, allow all to correlate

EF_0 ~~ EF_1
EF_0 ~~ EF_2
EF_0 ~~ EF_3
EF_1 ~~ EF_2
EF_1 ~~ EF_3
EF_2 ~~ EF_3


#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

'

mi.configural.fit<-lavaan(mi.configural, data=df, missing="fiml", estimator = "MLR") #add MLR for non-normality
summary(mi.configural.fit, fit.measures=T, standardized=T)



################ Metric ################
#are loadings equivalent across time?
#including labels for all indicators making them equal across waves
#removed constraints on factor variance

mi.metric<-'
#define latent variables, all loadings fixed to be equal across time
#lavaan only likes having one argument per variable, so you have to list the first indicator #twice to freely estimate the parameter (NA*) and then again to show that it is fixed across #waves (l1*). The actual manifest variable with the factor loading will only show up once in #the output!

EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS


#define indicator intercepts

DIGBACCT_W0 ~ 1
DIGBACCT_W1 ~ 1
DIGBACCT_W2 ~ 1
DIGBACCT_W3 ~ 1

UDSVERTN_W0S ~ 1
UDSVERTN_W1S ~ 1
UDSVERTN_W2S ~ 1
UDSVERTN_W3S ~ 1

TRAILB_W0rS ~ 1
TRAILB_W1rS ~ 1
TRAILB_W2rS ~ 1
TRAILB_W3rS ~ 1

#define indicator residual variance

DIGBACCT_W0 ~~ DIGBACCT_W0
DIGBACCT_W1 ~~ DIGBACCT_W1
DIGBACCT_W2 ~~ DIGBACCT_W2
DIGBACCT_W3 ~~ DIGBACCT_W3

UDSVERTN_W0S ~~ UDSVERTN_W0S
UDSVERTN_W1S ~~ UDSVERTN_W1S
UDSVERTN_W2S ~~ UDSVERTN_W2S
UDSVERTN_W3S ~~ UDSVERTN_W3S

TRAILB_W0rS ~~ TRAILB_W0rS
TRAILB_W1rS ~~ TRAILB_W1rS
TRAILB_W2rS ~~ TRAILB_W2rS
TRAILB_W3rS ~~ TRAILB_W3rS

#specify latent means and variance
#fix mean of latent factors to 0 (creates standardized latent factor)

EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix variance of first factor to 1, remove constraints for rest (creates standardized baseline factor)

EF_0 ~~ (1)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#specify latent correlations, allow all to correlate

EF_0 ~~ EF_1
EF_0 ~~ EF_2
EF_0 ~~ EF_3
EF_1 ~~ EF_2
EF_1 ~~ EF_3
EF_2 ~~ EF_3


#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

'

mi.metric.fit<-lavaan(mi.metric,data=df, missing="fiml", estimator = "MLR")
summary(mi.metric.fit, fit.measures=T, standardized=T)


#did imposing this constraint significantly worsen model fit?  (extract scaled values manually)
#chi square difference test bc these are nested models, use robust fit statistics
scaled_chi_squared_configural <- fitmeasures(mi.configural.fit, "chisq.scaled")
scaled_chi_squared_metric <- fitmeasures(mi.metric.fit, "chisq.scaled")
# Calculate the chi-squared difference
scaled_chi_squared_difference <- scaled_chi_squared_configural - scaled_chi_squared_metric
print(scaled_chi_squared_difference)
# Extract degrees of freedom for both models
df_configural <- fitmeasures(mi.configural.fit, "df")
df_metric <- fitmeasures(mi.metric.fit, "df")
# Calculate the difference in degrees of freedom
df_difference <- df_metric - df_configural
# Calculate the chi-squared difference
chi_squared_difference <- scaled_chi_squared_configural - scaled_chi_squared_metric

if (scaled_chi_squared_difference >= 0 && df_difference > 0) {
  p_value <- pchisq(scaled_chi_squared_difference, df_difference, lower.tail = FALSE)
  cat("P-value for chi-squared difference: ", p_value, "\n")
} else {
  cat("Invalid chi-squared or degrees of freedom differences: No valid p-value\n")
}

#difference in CFI using scaled values
fitmeasures(mi.metric.fit, c("cfi.scaled"))-fitmeasures(mi.configural.fit, c("cfi.scaled")) 

#for raw values (not scaled)
anova(mi.configural.fit,mi.metric.fit) 
fitmeasures(mi.metric.fit, c("cfi"))-fitmeasures(mi.configural.fit, c("cfi"))
 

################ Scalar ################
#are intercepts equivalent across time?
#if so, we can say something about mean structure of latent factor
#fix indicators to be equivalent across time, no other changes to model

mi.scalar<-'
#define latent variables, all loadings fixed to be equal across time
#lavaan only likes having one argument per variable, so you have to list the first indicator #twice to freely estimate the parameter (NA*) and then again to show that it is fixed across #waves (l1*). The actual manifest variable with the factor loading will only show up once in #the output!

EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS


#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance

DIGBACCT_W0 ~~ DIGBACCT_W0
DIGBACCT_W1 ~~ DIGBACCT_W1
DIGBACCT_W2 ~~ DIGBACCT_W2
DIGBACCT_W3 ~~ DIGBACCT_W3

UDSVERTN_W0S ~~ UDSVERTN_W0S
UDSVERTN_W1S ~~ UDSVERTN_W1S
UDSVERTN_W2S ~~ UDSVERTN_W2S
UDSVERTN_W3S ~~ UDSVERTN_W3S

TRAILB_W0rS ~~ TRAILB_W0rS
TRAILB_W1rS ~~ TRAILB_W1rS
TRAILB_W2rS ~~ TRAILB_W2rS
TRAILB_W3rS ~~ TRAILB_W3rS

#specify latent means and variance
#fix mean of the first latent factor to 0, remove constraints for rest (creates standardized baseline factor)

EF_0 ~ (0)*1
EF_1 ~ 1
EF_2 ~ 1
EF_3 ~ 1

#fix variance of first factor to 1, remove constraints for rest (creates standardized baseline factor)

EF_0 ~~ (1)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#With this specification, you can interpret the latent factor mean/var at the subsequent time #points based on wave0 mean/SD. So, if the mean for t1 is .25, that implies that the #participants scored .25 baseline SD units higher than baseline.

#specify latent correlations, allow all to correlate

EF_0 ~~ EF_1
EF_0 ~~ EF_2
EF_0 ~~ EF_3
EF_1 ~~ EF_2
EF_1 ~~ EF_3
EF_2 ~~ EF_3


#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

'

mi.scalar.fit<-lavaan(mi.scalar,data=df, missing="fiml", estimator = "MLR")
summary(mi.scalar.fit, fit.measures=T, standardized=T)




#did imposing scalar invariance significantly worsen model fit? (extract scaled values manually)
# Function to compare three models based on scaled chi-squared values
compare_models <- function(configural_fit, metric_fit, scalar_fit) {
  
  # Step 1: Extract the scaled chi-squared values and degrees of freedom
  chisq_configural <- fitmeasures(configural_fit, "chisq.scaled")
  chisq_metric <- fitmeasures(metric_fit, "chisq.scaled")
  chisq_scalar <- fitmeasures(scalar_fit, "chisq.scaled")

  df_configural <- fitmeasures(configural_fit, "df")
  df_metric <- fitmeasures(metric_fit, "df")
  df_scalar <- fitmeasures(scalar_fit, "df")
  
  # Step 2: Calculate chi-squared differences and degrees of freedom differences
  chisq_diff_configural_metric <- chisq_metric - chisq_configural 
  chisq_diff_metric_scalar <- chisq_scalar - chisq_metric 
  chisq_diff_configural_scalar <- chisq_scalar - chisq_configural 
  
  df_diff_configural_metric <-  df_metric - df_configural
  df_diff_metric_scalar <- df_scalar - df_metric 
  df_diff_configural_scalar <- df_scalar - df_configural 
  
  # Step 3: Print the differences for inspection
  cat("Chi-squared difference between configural and metric models: ", chisq_diff_configural_metric, "\n")
  cat("Degrees of freedom difference between configural and metric models: ", df_diff_configural_metric, "\n")
  
  cat("Chi-squared difference between metric and scalar models: ", chisq_diff_metric_scalar, "\n")
  cat("Degrees of freedom difference between metric and scalar models: ", df_diff_metric_scalar, "\n")
  
  cat("Chi-squared difference between configural and scalar models: ", chisq_diff_configural_scalar, "\n")
  cat("Degrees of freedom difference between configural and scalar models: ", df_diff_configural_scalar, "\n")
  
  # Step 4: Calculate p-values for each comparison using pchisq
  p_value_configural_metric <- if (chisq_diff_configural_metric >= 0 && df_diff_configural_metric > 0) {
    pchisq(chisq_diff_configural_metric, df_diff_configural_metric, lower.tail = FALSE)
  } else {
    NA
  }
  
  p_value_metric_scalar <- if (chisq_diff_metric_scalar >= 0 && df_diff_metric_scalar > 0) {
    pchisq(chisq_diff_metric_scalar, df_diff_metric_scalar, lower.tail = FALSE)
  } else {
    NA
  }
  
  p_value_configural_scalar <- if (chisq_diff_configural_scalar >= 0 && df_diff_configural_scalar > 0) {
    pchisq(chisq_diff_configural_scalar, df_diff_configural_scalar, lower.tail = FALSE)
  } else {
    NA
  }
  
  # Step 5: Print the p-values
  cat("P-value for chi-squared difference between configural and metric models: ", p_value_configural_metric, "\n")
  cat("P-value for chi-squared difference between metric and scalar models: ", p_value_metric_scalar, "\n")
  cat("P-value for chi-squared difference between configural and scalar models: ", p_value_configural_scalar, "\n")
}

# Compare models:
compare_models(mi.configural.fit, mi.metric.fit, mi.scalar.fit)

# Compare scaled CFI
fitmeasures(mi.scalar.fit, c("cfi.scaled"))-fitmeasures(mi.metric.fit, c("cfi.scaled"))

## for raw values (not scaled)
anova(mi.configural.fit,mi.metric.fit,mi.scalar.fit) 
fitmeasures(mi.metric.fit, c("cfi"))-fitmeasures(mi.scalar.fit, c("cfi")) 


################ Strict ################
#are residual variances equivalent across time? 
#if so, ratio of specific variance in an indicator as well as measurement error is equivalent across time
#equated residual variances across indicators

mi.strict<-'
#define latent variables, all loadings fixed to be equal across time
#lavaan only likes having one argument per variable, so you have to list the first indicator #twice to freely estimate the parameter (NA*) and then again to show that it is fixed across #waves (l1*). The actual manifest variable with the factor loading will only show up once in #the output!

EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS


#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance, all variances fixed to be equal across time

DIGBACCT_W0 ~~ (v1)*DIGBACCT_W0
DIGBACCT_W1 ~~ (v1)*DIGBACCT_W1
DIGBACCT_W2 ~~ (v1)*DIGBACCT_W2
DIGBACCT_W3 ~~ (v1)*DIGBACCT_W3

UDSVERTN_W0S ~~ (v2)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (v2)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (v2)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (v2)*UDSVERTN_W3S

TRAILB_W0rS ~~ (v3)*TRAILB_W0rS
TRAILB_W1rS ~~ (v3)*TRAILB_W1rS
TRAILB_W2rS ~~ (v3)*TRAILB_W2rS
TRAILB_W3rS ~~ (v3)*TRAILB_W3rS

#specify latent means and variance
#fix mean of the first latent factor to 0, remove constraints for rest (creates standardized baseline factor)

EF_0 ~ (0)*1
EF_1 ~ 1
EF_2 ~ 1
EF_3 ~ 1

#fix variance of first factor to 1, remove constraints for rest (creates standardized baseline factor)

EF_0 ~~ (1)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#With this specification, you can interpret the latent factor mean/var at the subsequent time #points based on wave0 mean/SD. So, if the mean for t1 is .25, that implies that the #participants scored .25 baseline SD units higher than baseline.

#specify latent correlations, allow all to correlate

EF_0 ~~ EF_1
EF_0 ~~ EF_2
EF_0 ~~ EF_3
EF_1 ~~ EF_2
EF_1 ~~ EF_3
EF_2 ~~ EF_3


#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

'


mi.strict.fit<-lavaan(mi.strict,data=df, missing="fiml", estimator = "MLR")
summary(mi.strict.fit, fit.measures=T, standardized=T)


#did imposing strict invariance significantly worsen model fit?
#chi square difference test bc these are nested models, use robust fit statistics (extract scaled values manually)
# Function to compare four models based on scaled chi-squared values
# Function to compare four models based on scaled chi-squared values
compare_four_models <- function(configural_fit, metric_fit, scalar_fit, strict_fit) {
  
  # Step 1: Extract the scaled chi-squared values and degrees of freedom
  chisq_configural <- fitmeasures(configural_fit, "chisq.scaled")
  chisq_metric <- fitmeasures(metric_fit, "chisq.scaled")
  chisq_scalar <- fitmeasures(scalar_fit, "chisq.scaled")
  chisq_strict <- fitmeasures(strict_fit, "chisq.scaled")
  
  df_configural <- fitmeasures(configural_fit, "df")
  df_metric <- fitmeasures(metric_fit, "df")
  df_scalar <- fitmeasures(scalar_fit, "df")
  df_strict <- fitmeasures(strict_fit, "df")
  
  # Step 2: Calculate chi-squared differences and degrees of freedom differences
  chisq_diff_configural_metric <- chisq_metric - chisq_configural 
  chisq_diff_metric_scalar <- chisq_scalar- chisq_metric
  chisq_diff_scalar_strict <- chisq_strict - chisq_scalar
  chisq_diff_configural_scalar <- chisq_scalar - chisq_configural
  chisq_diff_configural_strict <- chisq_strict - chisq_configural
  chisq_diff_metric_strict <- chisq_strict - chisq_metric
  
  df_diff_configural_metric <- df_metric - df_configural 
  df_diff_metric_scalar <- df_scalar- df_metric
  df_diff_scalar_strict <- df_strict - df_scalar
  df_diff_configural_scalar <- df_scalar - df_configural
  df_diff_configural_strict <- df_strict - df_configural
  df_diff_metric_strict <- df_strict - df_metric
  
  # Step 3: Print the differences for inspection
  cat("Chi-squared difference between configural and metric models: ", chisq_diff_configural_metric, "\n")
  cat("Degrees of freedom difference between configural and metric models: ", df_diff_configural_metric, "\n")
  
  cat("Chi-squared difference between metric and scalar models: ", chisq_diff_metric_scalar, "\n")
  cat("Degrees of freedom difference between metric and scalar models: ", df_diff_metric_scalar, "\n")
  
  cat("Chi-squared difference between scalar and strict models: ", chisq_diff_scalar_strict, "\n")
  cat("Degrees of freedom difference between scalar and strict models: ", df_diff_scalar_strict, "\n")
  
  cat("Chi-squared difference between configural and scalar models: ", chisq_diff_configural_scalar, "\n")
  cat("Degrees of freedom difference between configural and scalar models: ", df_diff_configural_scalar, "\n")
  
  cat("Chi-squared difference between configural and strict models: ", chisq_diff_configural_strict, "\n")
  cat("Degrees of freedom difference between configural and strict models: ", df_diff_configural_strict, "\n")
  
  cat("Chi-squared difference between metric and strict models: ", chisq_diff_metric_strict, "\n")
  cat("Degrees of freedom difference between metric and strict models: ", df_diff_metric_strict, "\n")
  
  # Step 4: Check for negative or invalid differences before calculating p-value
  check_and_calculate_p_value <- function(chisq_diff, df_diff) {
    if (chisq_diff < 0 || df_diff <= 0) {
      return(NA) # Invalid difference
    } else {
      return(pchisq(chisq_diff, df_diff, lower.tail = FALSE))
    }
  }
  
  # Step 5: Calculate p-values for each comparison
  p_value_configural_metric <- check_and_calculate_p_value(chisq_diff_configural_metric, df_diff_configural_metric)
  p_value_metric_scalar <- check_and_calculate_p_value(chisq_diff_metric_scalar, df_diff_metric_scalar)
  p_value_scalar_strict <- check_and_calculate_p_value(chisq_diff_scalar_strict, df_diff_scalar_strict)
  p_value_configural_scalar <- check_and_calculate_p_value(chisq_diff_configural_scalar, df_diff_configural_scalar)
  p_value_configural_strict <- check_and_calculate_p_value(chisq_diff_configural_strict, df_diff_configural_strict)
  p_value_metric_strict <- check_and_calculate_p_value(chisq_diff_metric_strict, df_diff_metric_strict)
  
  # Step 6: Print the p-values
  cat("P-value for chi-squared difference between configural and metric models: ", p_value_configural_metric, "\n")
  cat("P-value for chi-squared difference between metric and scalar models: ", p_value_metric_scalar, "\n")
  cat("P-value for chi-squared difference between scalar and strict models: ", p_value_scalar_strict, "\n")
  cat("P-value for chi-squared difference between configural and scalar models: ", p_value_configural_scalar, "\n")
  cat("P-value for chi-squared difference between configural and strict models: ", p_value_configural_strict, "\n")
  cat("P-value for chi-squared difference between metric and strict models: ", p_value_metric_strict, "\n")
}

# Compare models
compare_four_models(mi.configural.fit, mi.metric.fit, mi.scalar.fit, mi.strict.fit)
# Compare scaled CFI
fitmeasures(mi.strict.fit, c("cfi.scaled"))-fitmeasures(mi.scalar.fit, c("cfi.scaled"))

### for raw values (not scaled)
anova(mi.configural.fit,mi.metric.fit,mi.scalar.fit,mi.strict.fit)
fitmeasures(mi.strict.fit, c("cfi"))-fitmeasures(mi.scalar.fit, c("cfi"))




### Growth Curve Analysis, Curve of Factors Model
################ CUFF Test Model 1: Variances Fixed to 0 ################
#Curve of Factors
cuff.vars.0S <-'
#define latent variables
EF_0 =~ (ldigbacc)*DIGBACCT_W0 + (llett)*UDSVERTN_W0S + (ltrail)*TRAILB_W0rS
EF_1 =~ (ldigbacc)*DIGBACCT_W1 + (llett)*UDSVERTN_W1S + (ltrail)*TRAILB_W1rS
EF_2 =~ (ldigbacc)*DIGBACCT_W2 + (llett)*UDSVERTN_W2S + (ltrail)*TRAILB_W2rS
EF_3 =~ (ldigbacc)*DIGBACCT_W3 + (llett)*UDSVERTN_W3S + (ltrail)*TRAILB_W3rS

#define indicator intercepts, 

DIGBACCT_W0 ~ (idigbacc)*1
DIGBACCT_W1 ~ (idigbacc)*1
DIGBACCT_W2 ~ (idigbacc)*1
DIGBACCT_W3 ~ (idigbacc)*1

UDSVERTN_W0S ~ (ilett)*1
UDSVERTN_W1S ~ (ilett)*1
UDSVERTN_W2S ~ (ilett)*1
UDSVERTN_W3S ~ (ilett)*1

TRAILB_W0rS ~ (itrail)*1
TRAILB_W1rS ~ (itrail)*1
TRAILB_W2rS ~ (itrail)*1
TRAILB_W3rS ~ (itrail)*1

#define indicator residual variance, applying strict MI

DIGBACCT_W0 ~~ (varn)*DIGBACCT_W0
DIGBACCT_W1 ~~ (varn)*DIGBACCT_W1
DIGBACCT_W2 ~~ (varn)*DIGBACCT_W2
DIGBACCT_W3 ~~ (varn)*DIGBACCT_W3

UDSVERTN_W0S ~~ (varl)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (varl)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (varl)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (varl)*UDSVERTN_W3S

TRAILB_W0rS ~~ (vart)*TRAILB_W0rS
TRAILB_W1rS ~~ (vart)*TRAILB_W1rS
TRAILB_W2rS ~~ (vart)*TRAILB_W2rS
TRAILB_W3rS ~~ (vart)*TRAILB_W3rS

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

#fix EF variance to 0 for model specificity

EF_0 ~~ (0)*EF_0
EF_1 ~~ (0)*EF_1
EF_2 ~~ (0)*EF_2
EF_3 ~~ (0)*EF_3


i ~~ 1*i
s ~~ s

i ~~ s
'


cuff.vars.0.fitS<-lavaan(cuff.vars.0S,data=df, missing="fiml", estimator = "MLR")
summary(cuff.vars.0.fitS, fit.measures=T, standardized=T)


################ CUFF Test Model 2: Equated Variance Estimates ################
#Curve of Factors
cuff.vars.eqT <- '
#define latent variables
EF_0 =~ (ldigbacc)*DIGBACCT_W0 + (llett)*UDSVERTN_W0S + (ltrail)*TRAILB_W0rS
EF_1 =~ (ldigbacc)*DIGBACCT_W1 + (llett)*UDSVERTN_W1S + (ltrail)*TRAILB_W1rS
EF_2 =~ (ldigbacc)*DIGBACCT_W2 + (llett)*UDSVERTN_W2S + (ltrail)*TRAILB_W2rS
EF_3 =~ (ldigbacc)*DIGBACCT_W3 + (llett)*UDSVERTN_W3S + (ltrail)*TRAILB_W3rS

#define indicator intercepts, 

DIGBACCT_W0 ~ (idigbacc)*1
DIGBACCT_W1 ~ (idigbacc)*1
DIGBACCT_W2 ~ (idigbacc)*1
DIGBACCT_W3 ~ (idigbacc)*1

UDSVERTN_W0S ~ (ilett)*1
UDSVERTN_W1S ~ (ilett)*1
UDSVERTN_W2S ~ (ilett)*1
UDSVERTN_W3S ~ (ilett)*1

TRAILB_W0rS ~ (itrail)*1
TRAILB_W1rS ~ (itrail)*1
TRAILB_W2rS ~ (itrail)*1
TRAILB_W3rS ~ (itrail)*1

#define indicator residual variance, applying strict MI

DIGBACCT_W0 ~~ (varn)*DIGBACCT_W0
DIGBACCT_W1 ~~ (varn)*DIGBACCT_W1
DIGBACCT_W2 ~~ (varn)*DIGBACCT_W2
DIGBACCT_W3 ~~ (varn)*DIGBACCT_W3

UDSVERTN_W0S ~~ (varl)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (varl)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (varl)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (varl)*UDSVERTN_W3S

TRAILB_W0rS ~~ (vart)*TRAILB_W0rS
TRAILB_W1rS ~~ (vart)*TRAILB_W1rS
TRAILB_W2rS ~~ (vart)*TRAILB_W2rS
TRAILB_W3rS ~~ (vart)*TRAILB_W3rS

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

#fix variance of EF to be equal and set it to > .001 so it wont be neg

EF_0 ~~ (varE)*EF_0
EF_1 ~~ (varE)*EF_1
EF_2 ~~ (varE)*EF_2
EF_3 ~~ (varE)*EF_3

varE > .001

i ~~ 1*i
s ~~ s

i ~~ s
'


cuff.vars.eqT.fit<-lavaan(cuff.vars.eqT,data=df, missing="fiml", estimator = "MLR")
summary(cuff.vars.eqT.fit, fit.measures=T, standardized=T)


#did imposing this constraint significantly worsen model fit? 
#chi square difference test bc these are nested models, use robust fit statistics (extract scaled manually)
scaled_chi_squared_cuff.vars.eqT.fit <- fitmeasures(cuff.vars.eqT.fit, "chisq.scaled")
scaled_chi_squared_cuff.vars.0.fitS <- fitmeasures(cuff.vars.0.fitS, "chisq.scaled")
# Calculate the chi-squared difference
chi_squared_difference_01 <- scaled_chi_squared_cuff.vars.0.fitS - scaled_chi_squared_cuff.vars.eqT.fit
print(chi_squared_difference_01)

# Extract degrees of freedom for both models
df_cuff.vars.eqT.fit <- fitmeasures(cuff.vars.eqT.fit, "df")
df_cuff.vars.0.fitS <- fitmeasures(cuff.vars.0.fitS, "df")
# Calculate the difference in degrees of freedom
df_difference_01 <- df_cuff.vars.0.fitS - df_cuff.vars.eqT.fit
# Calculate the chi-squared difference
chi_squared_difference_01 <- scaled_chi_squared_cuff.vars.0.fitS - scaled_chi_squared_cuff.vars.eqT.fit

if (chi_squared_difference_01 >= 0 && df_difference_01 > 0) {
  p_value <- pchisq(chi_squared_difference_01, df_difference_01, lower.tail = FALSE)
  cat("P-value for chi-squared difference: ", p_value, "\n")
} else {
  cat("Invalid chi-squared or degrees of freedom differences: No valid p-value\n")
}
fitmeasures(cuff.vars.eqT.fit, c("cfi.scaled"))-fitmeasures(cuff.vars.0.fitS, c("cfi.scaled"))
#if difference in chisq > .01 then models are not fitting equivalently


# non scaled chi square difference test 
anova(cuff.vars.eqT.fit,cuff.vars.0.fitS)

################ CUFF Test Model 3: Constraining Variances to be Positive ################
#Curve of Factors
cuff.vars.02 <- '
#define latent variables
EF_0 =~ (ldigbacc)*DIGBACCT_W0 + (llett)*UDSVERTN_W0S + (ltrail)*TRAILB_W0rS
EF_1 =~ (ldigbacc)*DIGBACCT_W1 + (llett)*UDSVERTN_W1S + (ltrail)*TRAILB_W1rS
EF_2 =~ (ldigbacc)*DIGBACCT_W2 + (llett)*UDSVERTN_W2S + (ltrail)*TRAILB_W2rS
EF_3 =~ (ldigbacc)*DIGBACCT_W3 + (llett)*UDSVERTN_W3S + (ltrail)*TRAILB_W3rS

#define indicator intercepts, 

DIGBACCT_W0 ~ (idigbacc)*1
DIGBACCT_W1 ~ (idigbacc)*1
DIGBACCT_W2 ~ (idigbacc)*1
DIGBACCT_W3 ~ (idigbacc)*1

UDSVERTN_W0S ~ (ilett)*1
UDSVERTN_W1S ~ (ilett)*1
UDSVERTN_W2S ~ (ilett)*1
UDSVERTN_W3S ~ (ilett)*1

TRAILB_W0rS ~ (itrail)*1
TRAILB_W1rS ~ (itrail)*1
TRAILB_W2rS ~ (itrail)*1
TRAILB_W3rS ~ (itrail)*1

#define indicator residual variance, applying strict MI bc we established that earlier

DIGBACCT_W0 ~~ (varn)*DIGBACCT_W0
DIGBACCT_W1 ~~ (varn)*DIGBACCT_W1
DIGBACCT_W2 ~~ (varn)*DIGBACCT_W2
DIGBACCT_W3 ~~ (varn)*DIGBACCT_W3

UDSVERTN_W0S ~~ (varl)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (varl)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (varl)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (varl)*UDSVERTN_W3S

TRAILB_W0rS ~~ (vart)*TRAILB_W0rS
TRAILB_W1rS ~~ (vart)*TRAILB_W1rS
TRAILB_W2rS ~~ (vart)*TRAILB_W2rS
TRAILB_W3rS ~~ (vart)*TRAILB_W3rS

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

#fix variance of EF 0 and 2 and set to > .0001

EF_0 ~~ (varE0)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

varE0 > .0001

i ~~ 1*i
s ~~ s

i ~~ s
'


cuff.vars.02.fit<-lavaan(cuff.vars.02,data=df, missing="fiml", estimator = "MLR")
summary(cuff.vars.02.fit, fit.measures=T, standardized=T)


#did imposing this constraint significantly worsen model fit? 
#chi square difference test bc these are nested models, use robust fit statistics (extract scaled manually)
scaled_chi_squared_cuff.vars.eqT.fit <- fitmeasures(cuff.vars.eqT.fit, "chisq.scaled")
scaled_chi_squared_cuff.vars.02.fit <- fitmeasures(cuff.vars.02.fit, "chisq.scaled")
# Calculate the chi-squared difference
chi_squared_difference_02 <-  scaled_chi_squared_cuff.vars.eqT.fit - scaled_chi_squared_cuff.vars.02.fit
print(chi_squared_difference_02)

# Extract degrees of freedom for both models
df_cuff.vars.eqT.fit <- fitmeasures(cuff.vars.eqT.fit, "df")
df_cuff.vars.02.fit <- fitmeasures(cuff.vars.02.fit, "df")
# Calculate the difference in degrees of freedom
df_difference_02 <-  df_cuff.vars.eqT.fit - df_cuff.vars.02.fit


if (chi_squared_difference_02 >= 0 && df_difference_02 > 0) {
  p_value <- pchisq(chi_squared_difference_02, df_difference_02, lower.tail = FALSE)
  cat("P-value for chi-squared difference: ", p_value, "\n")
} else {
  cat("Invalid chi-squared or degrees of freedom differences: No valid p-value\n")
}

fitmeasures(cuff.vars.02.fit, c("cfi.scaled"))-fitmeasures(cuff.vars.eqT.fit, c("cfi.scaled"))
#if difference in chisq > .01 then models are not fitting equivalently


#non scaled
anova(cuff.vars.eqT.fit,cuff.vars.0.fitS,cuff.vars.02.fit) 


################ CUFF Test Model 4: Final Model ################
#Curve of Factors

cuff.vars.0.02 <- '
#define latent variables, all loadings fixed to be equal across time
#lavaan only likes having one argument per variable, so you have to list the first indicator #twice to freely estimate the parameter (NA*) and then again to show that it is fixed across #waves (l1*). The actual manifest variable with the factor loading will only show up once in #the output!

EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS


#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance, all variances fixed to be equal across time

DIGBACCT_W0 ~~ (v1)*DIGBACCT_W0
DIGBACCT_W1 ~~ (v1)*DIGBACCT_W1
DIGBACCT_W2 ~~ (v1)*DIGBACCT_W2
DIGBACCT_W3 ~~ (v1)*DIGBACCT_W3

UDSVERTN_W0S ~~ (v2)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (v2)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (v2)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (v2)*UDSVERTN_W3S

TRAILB_W0rS ~~ (v3)*TRAILB_W0rS
TRAILB_W1rS ~~ (v3)*TRAILB_W1rS
TRAILB_W2rS ~~ (v3)*TRAILB_W2rS
TRAILB_W3rS ~~ (v3)*TRAILB_W3rS

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix EF 0 and 2 variance to 0 for model specificity

EF_0 ~~ (0)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS 

#Growth curve

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3

#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1


i ~~ 1*i
s ~~ s

i ~~ s

'

cuff.vars.0.02.fit<-lavaan(cuff.vars.0.02,data=df, missing="fiml", estimator = "MLR")
summary(cuff.vars.0.02.fit, fit.measures=T, standardized=T)


#did imposing this constraint significantly worsen model fit? 
#chi square difference test bc these are nested models, use robust fit statistics (extract scaled manually)
scaled_chi_squared_cuff.vars.02.fit <- fitmeasures(cuff.vars.02.fit, "chisq.scaled")
scaled_chi_squared_cuff.vars.0.02.fit <- fitmeasures(cuff.vars.0.02.fit, "chisq.scaled")
# Calculate the chi-squared difference
chi_squared_difference_03 <-  scaled_chi_squared_cuff.vars.0.02.fit - scaled_chi_squared_cuff.vars.02.fit
print(chi_squared_difference_03)

# Extract degrees of freedom for both models
df_cuff.vars.02.fit <- fitmeasures(cuff.vars.02.fit, "df")
df_cuff.vars.0.02.fit <- fitmeasures(cuff.vars.0.02.fit, "df")
# Calculate the difference in degrees of freedom
df_difference_03 <-  df_cuff.vars.0.02.fit - df_cuff.vars.02.fit


if (chi_squared_difference_03 >= 0 && df_difference_03 > 0) {
  p_value <- pchisq(chi_squared_difference_03, df_difference_03, lower.tail = FALSE)
  cat("P-value for chi-squared difference: ", p_value, "\n")
} else {
  cat("Invalid chi-squared or degrees of freedom differences: No valid p-value\n")
}
fitmeasures(cuff.vars.0.02.fit, c("cfi.scaled"))-fitmeasures(cuff.vars.02.fit, c("cfi.scaled"))


# not scaled
anova(cuff.vars.eqT.fit,cuff.vars.0.fitS,cuff.vars.02.fit,cuff.vars.0.02.fit) 


### Predicted CUFF
################ Regress NPS on EF CUFF Model ################
#First, reverse code NPIQ severity to make everything same direction
#possible range of scores is 0 - 36 (12 sx * 3 severity)

df = 
  df %>%
  mutate(NPIQ_total_s_W0r = 36 - NPIQ_total_s_W0) 
#relocate variable
df = df %>% 
  relocate(NPIQ_total_s_W0r, .after = NPIQ_total_s_W0)
#check
cor.test(df$NPIQ_total_s_W0, df$NPIQ_total_s_W0r, 
         method = "pearson")

mean(df$NPIQ_total_s_W0r)
sd(df$NPIQ_total_s_W0r)

#Curve of Factors predicted by wave 0 NPIQ
cuff_pred <-'
#define latent variables
EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS

#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance, all variances fixed to be equal across time

DIGBACCT_W0 ~~ (v1)*DIGBACCT_W0
DIGBACCT_W1 ~~ (v1)*DIGBACCT_W1
DIGBACCT_W2 ~~ (v1)*DIGBACCT_W2
DIGBACCT_W3 ~~ (v1)*DIGBACCT_W3

UDSVERTN_W0S ~~ (v2)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (v2)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (v2)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (v2)*UDSVERTN_W3S

TRAILB_W0rS ~~ (v3)*TRAILB_W0rS
TRAILB_W1rS ~~ (v3)*TRAILB_W1rS
TRAILB_W2rS ~~ (v3)*TRAILB_W2rS
TRAILB_W3rS ~~ (v3)*TRAILB_W3rS

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix variance of EF0 and 2 to be 0 so they wont be neg

EF_0 ~~ (0)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS

#Growth curve

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 


#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

i ~~ 1*i
s ~~ s

i ~~ s

#add NPIQ predictor
#regress intercept and slope on NPIQ severity at wave 1
i ~ NPIQ_total_s_W0r
s ~ NPIQ_total_s_W0r
'


cuff_pred.fit<-lavaan(cuff_pred,data=df, missing="fiml", estimator = "MLR")
summary(cuff_pred.fit, fit.measures=T, standardized=T)







#did adding the predictor significantly worsen model fit? 
fitmeasures(cuff_pred.fit, c("cfi.scaled"))-fitmeasures(cuff_pred.fit, c("cfi.scaled"))
#if difference in chisq > .01 then models are not fitting equivalently

standardizedSolution(cuff_pred.fit, ci = T) #extract standardized values



### Post Hoc Tests
################ Post Hoc 1: Winsorize NPS Values################
#Step 1: Windsorize the NPS scores that are >3SD from the mean to closest observed value
#pulls in the tail so the extreme scores are not having outsized influence on results

#create a new NPS variable that has been windsorized. If the participant's NPS score is MORE than 3 SD of NPS above the mean, then replace their score with a value of 3 SD above the mean. If not, then just leave the data as it was.

df$NPS_windsorize <- ifelse(df$NPIQ_total_s_W0 > (mean(df$NPIQ_total_s_W0))+(3*sd(df$NPIQ_total_s_W0)), (mean(df$NPIQ_total_s_W0))+(3*sd(df$NPIQ_total_s_W0)), df$NPIQ_total_s_W0)

#check distribution
#NPIQ severity

mean(df$NPIQ_total_s_W0)
mean(df$NPS_windsorize)
sd(df$NPIQ_total_s_W0)
sd(df$NPS_windsorize)
print(skewness(df$NPIQ_total_s_W0))
print(skewness(df$NPS_windsorize))
print(kurtosis(df$NPIQ_total_s_W0))
print(kurtosis(df$NPS_windsorize))
hist(df$NPIQ_total_s_W0)
hist(df$NPS_windsorize)

#reverse score the windsorized variable
#identify range of windsorized variable
describe(df$NPS_windsorize) #min 0, max 14.3

df = 
  df %>%
  mutate(NPS_windsorizeR = 14.3 - NPS_windsorize) 
#relocate variable
df = df %>% 
  relocate(NPS_windsorizeR, .after = NPS_windsorize)
#check
cor.test(df$NPS_windsorize, df$NPS_windsorizeR, 
         method = "pearson")

#test model using windsorized variable
#Curve of Factors predicted by wave 0 NPIQ windsorized
cuff_pred_windsor <-'
#define latent variables
EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS

#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance, all variances fixed to be equal across time

DIGBACCT_W0 ~~ (v1)*DIGBACCT_W0
DIGBACCT_W1 ~~ (v1)*DIGBACCT_W1
DIGBACCT_W2 ~~ (v1)*DIGBACCT_W2
DIGBACCT_W3 ~~ (v1)*DIGBACCT_W3

UDSVERTN_W0S ~~ (v2)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (v2)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (v2)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (v2)*UDSVERTN_W3S

TRAILB_W0rS ~~ (v3)*TRAILB_W0rS
TRAILB_W1rS ~~ (v3)*TRAILB_W1rS
TRAILB_W2rS ~~ (v3)*TRAILB_W2rS
TRAILB_W3rS ~~ (v3)*TRAILB_W3rS

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix variance of EF0 and 2 to be 0

EF_0 ~~ (0)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS

#Growth curve

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 


#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

i ~~ 1*i
s ~~ s

i ~~ s

#add NPIQ predictor
#regress intercept and slope on NPIQ severity at wave 1
i ~ NPS_windsorizeR
s ~ NPS_windsorizeR
'


cuff_pred_windsor.fit<-lavaan(cuff_pred_windsor,data=df, missing="fiml", estimator = "MLR")
summary(cuff_pred_windsor.fit, fit.measures=T, standardized=T)


################ Post Hoc 2: Quanties################
#check that association is linear
#creates a new variable called NPS_cat based on taking 4 equally sized groups. 

df$NPS_cat<-quantcut(df$NPIQ_total_s_W0, q = 4)

#Create dummy codes with absent as reference group
df$NPS_cat_absent <- ifelse(df$NPS_cat == "0", 0, 0)
df$NPS_cat_mild <- ifelse(df$NPS_cat == "(0,2]", 1, 0)
df$NPS_cat_mod <- ifelse(df$NPS_cat == "(2,4]", 1, 0)
df$NPS_cat_sev <- ifelse(df$NPS_cat == "(4,26]", 1, 0)

median(df$NPIQ_total_s_W0)
describe(df$NPIQ_total_s_W0)

#now enter these in the model
#if regression coefficients get linearly bigger across the categories, then linear specification is met

#Curve of Factors predicted by wave 0 NPIQ quart dummies
cuff_pred_quart <-'
#define latent variables
EF_0 =~ NA*DIGBACCT_W0 + (l1)*DIGBACCT_W0 + (l2)*UDSVERTN_W0S + (l3)*TRAILB_W0rS
EF_1 =~ NA*DIGBACCT_W1 + (l1)*DIGBACCT_W1 + (l2)*UDSVERTN_W1S + (l3)*TRAILB_W1rS
EF_2 =~ NA*DIGBACCT_W2 + (l1)*DIGBACCT_W2 + (l2)*UDSVERTN_W2S + (l3)*TRAILB_W2rS
EF_3 =~ NA*DIGBACCT_W3 + (l1)*DIGBACCT_W3 + (l2)*UDSVERTN_W3S + (l3)*TRAILB_W3rS

#define indicator intercepts, all intercepts fixed to be equal across time

DIGBACCT_W0 ~ (i1)*1
DIGBACCT_W1 ~ (i1)*1
DIGBACCT_W2 ~ (i1)*1
DIGBACCT_W3 ~ (i1)*1

UDSVERTN_W0S ~ (i2)*1
UDSVERTN_W1S ~ (i2)*1
UDSVERTN_W2S ~ (i2)*1
UDSVERTN_W3S ~ (i2)*1

TRAILB_W0rS ~ (i3)*1
TRAILB_W1rS ~ (i3)*1
TRAILB_W2rS ~ (i3)*1
TRAILB_W3rS ~ (i3)*1

#define indicator residual variance, all variances fixed to be equal across time

DIGBACCT_W0 ~~ (v1)*DIGBACCT_W0
DIGBACCT_W1 ~~ (v1)*DIGBACCT_W1
DIGBACCT_W2 ~~ (v1)*DIGBACCT_W2
DIGBACCT_W3 ~~ (v1)*DIGBACCT_W3

UDSVERTN_W0S ~~ (v2)*UDSVERTN_W0S
UDSVERTN_W1S ~~ (v2)*UDSVERTN_W1S
UDSVERTN_W2S ~~ (v2)*UDSVERTN_W2S
UDSVERTN_W3S ~~ (v2)*UDSVERTN_W3S

TRAILB_W0rS ~~ (v3)*TRAILB_W0rS
TRAILB_W1rS ~~ (v3)*TRAILB_W1rS
TRAILB_W2rS ~~ (v3)*TRAILB_W2rS
TRAILB_W3rS ~~ (v3)*TRAILB_W3rS

#specify latent variable means and variances
#all latent factors do not have a mean, mean of 0
EF_0 ~ (0)*1
EF_1 ~ (0)*1
EF_2 ~ (0)*1
EF_3 ~ (0)*1

#fix variance of EF0 to be 0

EF_0 ~~ (0)*EF_0
EF_1 ~~ EF_1
EF_2 ~~ EF_2
EF_3 ~~ EF_3

#specify residual correlations
#allowing each indicator to correlate with each other over time, helps model fit

#correlate all numbers
DIGBACCT_W0 ~~ DIGBACCT_W1 + DIGBACCT_W2 + DIGBACCT_W3
DIGBACCT_W1 ~~ DIGBACCT_W2 + DIGBACCT_W3 
DIGBACCT_W2 ~~ DIGBACCT_W3 

#correlate all letters
UDSVERTN_W0S ~~ UDSVERTN_W1S + UDSVERTN_W2S + UDSVERTN_W3S 
UDSVERTN_W1S ~~ UDSVERTN_W2S + UDSVERTN_W3S
UDSVERTN_W2S ~~ UDSVERTN_W3S 

#correlate all trails
TRAILB_W0rS ~~ TRAILB_W1rS + TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W1rS ~~ TRAILB_W2rS + TRAILB_W3rS 
TRAILB_W2rS ~~ TRAILB_W3rS

#Growth curve

#specify growth parameters
i=~ 1*EF_0 + 1*EF_1 + 1*EF_2 + 1*EF_3 
s=~ 0*EF_0 + 1*EF_1 + 2*EF_2 + 3*EF_3 


#fix intercept mean/var to 0 and 1
#slope is freely estimated
i ~ 0*1
s ~ 1

i ~~ 1*i
s ~~ s

i ~~ s

#add NPIQ predictor
#regress intercept and slope on NPIQ severity at wave 1
i ~ NPS_cat_mild + NPS_cat_mod + NPS_cat_sev
s ~ NPS_cat_mild + NPS_cat_mod + NPS_cat_sev
'


cuff_pred_quart.fit<-lavaan(cuff_pred_quart,data=df, missing="fiml", estimator = "MLR")
summary(cuff_pred_quart.fit, fit.measures=T, standardized=T)


